/*---------------------------------------------------------------------------
BMP.CPP - Part of FAST Viewer project
Description: This file handles bmp format. At present, there are two
accessible functions: one displays bmp files on screen and the other reads in
header of a bmp file. This module  may be used in some other project as it is
an independent one. Just include "bmp.h" in that project.
-----------------------------------------------------------------------------
Accessible functions:
int show_bmp (char *name);
void read_bmp_header (FILE *dataFile, bmpHeaderType bmpHeader);
---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include "mode13h.h"


/*---------------------------------------------------------------------------
GLOBAL VARIABLES
---------------------------------------------------------------------------*/
struct bmpHeaderType
   {
   char magic1, magic2;		// identity 'BM'
   long fileSize;		// size of file in bytes
   long reserved;		// always 0
   long pixelOffset;		// offset of data
   long bmiSize;		// remaining size of header in file
   long cols, rows;		// width and height of image
   int  nPlanes;		// number of color planes
   int  bitsPerPixel;		// number of bits used to represent a pixel
   long compression, cmpSize;	// compressed image?
   long xScale, yScale;
   long colors, impColors;
   };




/*---------------------------------------------------------------------------
FUNCTION:	Reads header of a bmp file
INPUT:		Name of file and header reference variable
PROCESS:	Opens up the file and reads in its header using fread
OUTPUT:		None
---------------------------------------------------------------------------*/
void read_bmp_header (FILE *dataFile, bmpHeaderType &bmpHeader)
   {
   fread (&bmpHeader, sizeof (bmpHeaderType), 1, dataFile);
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Shows a bmp file on screen
INPUT:		Name of file to be shown
PROCESS:	Opens up the file, interpret its header and shows it on the
		screen. Uses routines defined in mode13h.cpp.
OUTPUT:		Error code
---------------------------------------------------------------------------*/
int show_bmp (char *name)
   {
   bmpHeaderType bmpHeader;		// to hold image information
   FILE *dataFile; 			// file pointer
   byte r, g, b;                        // red, green and blue for palette
   int x, y;				// coordinates of pixel to plot
   unsigned int width;			// width of the image

   dataFile = fopen (name, "rb");	// open the file
   if (dataFile == NULL) return 5;	// could not open it? that's an error

   read_bmp_header (dataFile, bmpHeader);	// and check for errors
   if (bmpHeader.nPlanes != 1)			// unsupported bmp
      { fclose (dataFile); return 1; }
   if (bmpHeader.bitsPerPixel != 8) 		// unsupported bmp
      { fclose (dataFile); return 2; }
   if (bmpHeader.compression != 0)    		// unsupported bmp
      { fclose (dataFile); return 3; }
   if (bmpHeader.magic1 != 'B' || bmpHeader.magic2 != 'M')
      {fclose (dataFile); return 4;}		// invalid bmp

   fseek (dataFile, 54, SEEK_SET);	// start reading the image
   set_mcga();				// defined in mode13h.cpp
   for (x=0; x<256; x++)		// handle palette first
     {
     b = fgetc (dataFile); b = b>>2;	// colors are stored as bgr
     g = fgetc (dataFile); g = g>>2;
     r = fgetc (dataFile); r = r>>2;
     set_palette (x, r, g, b);		// defined in mode13h.cpp
     fgetc (dataFile);			// spare next byte
     }

   width = bmpHeader.cols;		// read width from the header
   while (width % 4 != 0) width++;	// make it a multiple of four
   for (y=bmpHeader.rows; y>0; y--)	// image is stored upside down
      for (x=0; x<width; x++)		// for whole width of image
	 put_pixel (x, y, fgetc(dataFile));	// read and plot

   fclose (dataFile);			// close the file
   getch();				// wait for a key press
   set_text();				// return to text mode: in mode13h.cpp
   return 0;				// reached here? there's no error
   }
/*-------------------------------------------------------------------------*/