/*---------------------------------------------------------------------------
GIF.H - Part of FAST Viewer project
Description: This file contains data types and proto types of functios defined
in gif.cpp. At present there are only two accessible functions: one displays
gif files and other reads information from a gif image.
-----------------------------------------------------------------------------
Accessible functions:
int show_gif (char *name);
void read_gif_info (FILE *dataFile, gifInfoType &gifInfo);
---------------------------------------------------------------------------*/


#include "mode13h.h"

/*---------------------------------------------------------------------------
ACCESSIBLE VARIABLES / DATA TYPES DEFINED IN GIF.CPP
---------------------------------------------------------------------------*/
struct gifInfoType		// our home made gif information header
   {
   char version[7];		// version either GIF89a or GIF87a
   byte gcTable;		// global color table exists? answer: 0 or 1
   byte bitsPerPixel;		// bits per pixel --> global
   byte background;		// usually 0
   unsigned int xStart, yStart; // usually 0, 0
   int width, height;		// local width and height of the image
   byte lcTable;		// local color table exists
   byte interlaced;		// interlaced image?
   };


/*---------------------------------------------------------------------------
ERROR MESSAGES
---------------------------------------------------------------------------*/
char *gifError[7] = {
	"Successful",
	"Unsupported GIF: Version - GIF89a",
	"Unsupported GIF: No global color table",
	"Unsupported GIF: Local color table found",
	"Unsupported GIF: Interlaced image",
	"Invalid GIF header",
	"Can't read this file"};


/*---------------------------------------------------------------------------
ACCESSIBLE PROTO TYPES
---------------------------------------------------------------------------*/
int show_gif (char *name);
int read_gif_info (FILE *dataFile, gifInfoType &gifInfo);
/*-------------------------------------------------------------------------*/