/*---------------------------------------------------------------------------
GUI.CPP - Part of FAST Viewer project
Description: This file contains general text mode graphical user interface
routines. It may be used in a different project as it is an independent
module. Just include "gui.h" in that project.
-----------------------------------------------------------------------------
Accessible variables:
. int far *textMem		// pointer to text memory 0xB8000000
-----------------------------------------------------------------------------
Accessible functions:
. void write (int x, int y, char *string, int attrib);
. void center_write (int row, char *string, int attrib);
. void draw_box (int left, int top, int width, int height, int attrib);
---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>


/*---------------------------------------------------------------------------
GLOBAL VARIABLES
---------------------------------------------------------------------------*/
#define COLMAX 80				// width of DOS mode window
int far *textMem = (int far *) 0xB8000000;	// pointer to text memory




/*---------------------------------------------------------------------------
FUNCTION:	Writes a string with given color at x, y
INPUT:		Location, attribute and the string
PROCESS:	Uses *textMem to put individual characters on screen
OUTPUT:		None
---------------------------------------------------------------------------*/
void write (int x,  int y, char *string, int attrib)
   {
   int i=0; x--; y--;
   while (string[i] != 0)
      * (textMem + y*COLMAX + x + i) = string[i++] | attrib;
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Writes a string with given color centered on screen
INPUT:		Row number, attribute and the string
PROCESS:	Uses *textMem to put individual characters on screen
OUTPUT:		None
---------------------------------------------------------------------------*/
void center_write (int row, char *string, int attrib)
   {
   int i=0, start; row--;
   start = (COLMAX - strlen (string)) / 2;
   while (string[i] != 0)
      * (textMem + row*COLMAX + start + i) = string[i++] | attrib;
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Draws a single lined box with given color, height & width
INPUT:		Box coordinates and attribute
PROCESS:	Uses *textMem to put individual ascii characters on screen
OUTPUT:		None
---------------------------------------------------------------------------*/
void draw_box (int left, int top, int width, int height, int attrib)
   {
   int i, j;

   top--; left--; width--; height--;
   * (textMem + top*COLMAX + left) = 218 | attrib; // 
   for (i=left+1; i<=left+width-1; i++)
      *(textMem+top*COLMAX+i) = 196 | attrib; // 
   * (textMem + top*COLMAX + left+width) = 191 | attrib; // 

   for (i=top+1; i<=top+height-1; i++)
      for (j=left; j<=left+width; j++)
	 if (j==left || j==left+width)
	    *(textMem+i*COLMAX+j) = 179 | attrib; // 
	 else *(textMem+i*COLMAX+j) = 32 | attrib;

   * (textMem+(top+height)*COLMAX+left) = 192 | attrib; // 
   for (i=left+1; i<=left+width; i++)
      *(textMem + (top+height)*COLMAX + i) = 196 | attrib; // 
   *(textMem + (top+height)*COLMAX+left+width) = 217 | attrib; // 
   }
/*-------------------------------------------------------------------------*/