/*---------------------------------------------------------------------------
PROJECT.CPP - Part of FAST Viewer project
Description: This file controls the interface and overall flow of the
program. Some of the functions called in this file are defined in bmp.cpp,
pcx.cpp, gif.cpp and gui.cpp. As the interface is text based, we don't need
mode13h.cpp here.
---------------------------------------------------------------------------*/

#include <stdio.h>
#include <conio.h>
#include <dir.h>
#include <dos.h>
#include <stdlib.h>
#include <string.h>
#include "gui.h"        // graphical user interface handling part
#include "bmp.h"	// bmp handling part
#include "pcx.h"	// pcx handling part
#include "gif.h"	// gif handling part

#define MAXFILES 64	// maximum number of files to display per screen


/*---------------------------------------------------------------------------
GLOBAL VARIABLES
---------------------------------------------------------------------------*/
struct filez { char name[13]; filez *next; }; 	// to sotre data of a file
struct filez *head;				// head of the linked list
int tFiles=0, top=1;				// total files to be displayed
						// number of file on the top

/*---------------------------------------------------------------------------
PROTOTYPES
---------------------------------------------------------------------------*/
void about (void);				// program information
void adjust_display (int selected, char key);   // highlights selected file
int  change_drive (int driverNr, char *path);	// change drive: 0=A, 1=B, ...
void draw_page (char *path);               	// draws the title page
void error (char *message);		  	// to show error message
void get_path (char *path);               	// get current path
void handle_file (char *path, int &selected, int info);	// handle select file
int  read_directory (char *path);   		// read directory from path
void show_directory (filez *head, int top, int tFiles);  // show directory
void show_file_info (char *filename, int type);	// show file information
int  support (ffblk afile);			// do we support this file?




/*---------------------------------------------------------------------------
FUNCTION:	Main
INPUT:		None
PROCESS:	Calls all other functions, controls the interface
OUTPUT:		None
---------------------------------------------------------------------------*/
void main (void)
   {
   char *path, key;			// path of current directory
   int selected=1;			// number of selected file
   int pos;
   _setcursortype (_NOCURSOR);     	// make cursor disappear
   get_path (path);                	// store current path in "path"
   draw_page(path);                	// draw title page
   tFiles = read_directory (path); 	// read the directory and store it
   show_directory (head, top, tFiles);  // show the stored directory
   adjust_display (selected, 0);   	// highlight first file name

   while ((key = getch()) != ESC)	// until ESC pressed
      {
      pos = selected-top+1;
      if (key == ENTER) handle_file (path, selected, 0);
      else if (key == 0) key = getch();	// read extended code
      switch (key)
	 {
	 case F1: about ();
		  draw_page (path);               	// draw title page
		  show_directory (head, top, tFiles);  // show the stored directory
		  adjust_display (1-top+selected, 0);  // highlight selected file name
		  break;

	 case F2: handle_file (path, selected, 1); break;

	 case ALTA: if (change_drive (0, path) == 0) selected = 1; break;
	 case ALTB: if (change_drive (1, path) == 0) selected = 1; break;
	 case ALTC: if (change_drive (2, path) == 0) selected = 1; break;
	 case ALTD: if (change_drive (3, path) == 0) selected = 1; break;
	 case ALTE: if (change_drive (4, path) == 0) selected = 1; break;

	 case RIGHT:			// right cursor key pressed?
	    if (selected+1 >= top+MAXFILES && selected < tFiles)
	       {
	       top += 4;
	       window (3, 3, 77, 21); clrscr();
	       show_directory (head, top, tFiles);
	       adjust_display (pos-3, 0);
	       selected++;
	       }
	    else if (tFiles > selected)
	       {
	       adjust_display (pos, key);
	       selected++;
	       }
	    break;

	 case LEFT:			// left cursor key pressed?
	    if (selected-1 < top && selected > 1)
	       {
	       top -= 4;
	       window (3, 3, 77, 21); clrscr();
	       show_directory (head, top, tFiles);
	       adjust_display (pos+3, 0);
	       selected--;
	       }
	    else if (selected > 1)
	       {
	       adjust_display (pos, key);
	       selected--;
	       }
	    break;

	 case DOWN:			// down cursor key pressed?
	    if (selected+4 >= top+MAXFILES && top+MAXFILES <= tFiles)
	       {
	       top += 4;
	       window (3, 3, 77, 21); clrscr();
	       show_directory (head, top, tFiles);
	       selected = (selected+4 < tFiles) ? selected+4 : tFiles;
	       pos = selected - top + 1;
	       adjust_display (pos, 0);
	       }
	    else if (tFiles >= selected+4)	// can we move down?
	       {
	       adjust_display (pos, key);	// ok, adjust display
	       selected += 4;			// selected file number
	       }
	    break;

	 case UP:			// up cursor key pressed?
	    if (selected-4 < top && selected-3 > 1)
	       {
	       top -= 4;
	       window (3, 3, 77, 21); clrscr();
	       show_directory (head, top, tFiles);
	       selected -= 4;
	       adjust_display (pos, 0);
	       }
	    else if (selected-3 > 1)	// can we move up?
	       {
	       adjust_display (pos, key);	// ok, adjust display
	       selected -= 4;			// selected file number
	       }
	    break;
	 }
      }

   // exiting
   _setcursortype (_NORMALCURSOR);	// make cursor appear again
   textcolor (LIGHTGRAY);		// reset foreground and
   textbackground (BLACK);		// background colors
   clrscr();				// clear the screen
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:       About
INPUT:		None
PROCESS:        Shows program info using draw_box and center_write
OUTPUT:		None
---------------------------------------------------------------------------*/
void about (void)
   {
   const int ABOUT_ATTRIB = 0x7100;		// blue on white
   draw_box (20, 8, 40, 10, ABOUT_ATTRIB);	// defined in gui.cpp
   center_write (8, "[ About ]", ABOUT_ATTRIB);		// program information
   center_write (10, "FAST Viewer", ABOUT_ATTRIB);
   center_write (11, "ICS Project Fall 98", ABOUT_ATTRIB);
   center_write (13, "Group Members", ABOUT_ATTRIB);
   center_write (14, "Khawaja Amin Ahmed (962)", ABOUT_ATTRIB);
   center_write (15, "Muhammad Ali Shah  (958)", ABOUT_ATTRIB);
   getch();					// wait for a key press
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Adjust display
INPUT:	        Select file number and key pressed
PROCESS:	Uses *textMem defined in gui.cpp to highlight the selected
		file
OUTPUT:		None
---------------------------------------------------------------------------*/
void adjust_display (int selected, char key)
   {
   const int SELECTED_ATTRIB = 0x7100;		// blue on white
   const int TEXT_ATTRIB     = 0x1E00;		// yellow on blue
   int size = 12,				// size to highlight
	i, position;				// lcv and pos of file name

   position = (selected-1) * 20 + 243;		// first file name at 243
   if (selected == 1 && key == 0)		// called to highlight first
      {                                         // file name?
      for (i=position; i<position+size; i++)
	 * (textMem+i) = (* (textMem+i) & 0x00FF) | SELECTED_ATTRIB;
      return;
      }


   // no, not called for the first time, dehighlight the previous selected
   for (i=position; i<position+size; i++)
      * (textMem+i) = (* (textMem+i) & 0x00FF) | TEXT_ATTRIB;

   switch (key)					// now check the key pressed
      {
      case RIGHT: position += 20; break;	// for right go 1 file ahead
      case LEFT : position -= 20; break;	// for left go 1 file back
      case DOWN : position += 20*4; break;	// for down go 4 files ahead
      case UP   : position -= 20*4; break;	// for up go 4 files back
      }

   for (i=position; i<position+size; i++)	// now highlight the new file
      * (textMem+i) = (* (textMem+i) & 0x00FF) | SELECTED_ATTRIB;
   }
/*-------------------------------------------------------------------------*/




int change_drive (int driveNr, char *path)
   {
   char newPath[MAXPATH];
   int pathLength;
   newPath[0] = 'A' + driveNr;
   strcpy (newPath+1, ":\\");  	// next 2 chars are : and \
   if (getcurdir (driveNr+1, newPath+3) == -1) return 1;

   strcpy (path, newPath);
   pathLength = strlen(path); 	// length of the path
   if (pathLength == 3) strcpy (path+pathLength, "*.*");   // root directory?
   else                 strcpy (path+pathLength, "\\*.*"); // no
   top = 1;
   draw_page (path);
   tFiles = read_directory (path);
   show_directory (head, top, tFiles);
   adjust_display (1, 0);
   return 0;
   }



/*---------------------------------------------------------------------------
FUNCTION:	Draw title page
INPUT:		Current path (read from get_path)
PROCESS:	Calls draw_box and center_write() to draw the title page of
		our program in text mode
OUTPUT:		None
---------------------------------------------------------------------------*/
void draw_page (char *path)
   {
   const int TITLE_ATTRIB = 0X1F00;	// white on blue
   const int MENU_ATTRIB  = 0X1B00;	// light cyan on blue

   window (1, 1, 80, 25);		// clear the screen
   textbackground (BLUE);		// with blue background
   clrscr();

   draw_box (1, 1, 80, 22, TITLE_ATTRIB); 		// draw a box
   center_write (1, "[ FAST Viewer ]", TITLE_ATTRIB); 	// title of the box
   center_write (22, "[ Copyright 1999 Amin And Ali ]", TITLE_ATTRIB);

   write (4, 24, "Cursor keys to move", MENU_ATTRIB); 	// display keys info
   write (4, 25, "Enter to view", MENU_ATTRIB);		// with light cyan on
   write (35, 24, "F1 for prog info", MENU_ATTRIB);	// blue background
   write (35, 25, "F2 for file info", MENU_ATTRIB);
   write (62, 24, "Esc to exit", MENU_ATTRIB);

   write (4, 2, path, TITLE_ATTRIB); 			// show path at top
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Error
INPUT:	        Message to print
PROCESS:	Uses center_write and draw box to show the error
OUTPUT:		None
---------------------------------------------------------------------------*/
void error (char *message)
   {
   const int TOP = 9;			// fixed top and height
   const int HEIGHT = 5;		// of the box
   const int TITLE_ATTRIB = 0X4F00; 	// white on red

   int left, length;			// variable left coordinate and width
   length = strlen (message);		// length of the message
   left = 40 - length / 2 - 2;		// compute left coordinate
   draw_box (left, TOP, length+6, HEIGHT, TITLE_ATTRIB);	// draw box
   center_write (TOP, "[ Error ]", TITLE_ATTRIB);		// heading
   center_write (TOP+2, message, TITLE_ATTRIB);			// message
   getch();							// wait...
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Get Path
INPUT:		Current Path (actually NULL)
PROCESS:	Calls getcurdir() and getdisk() to read current path,
		adds *.* at the end
OUTPUT:		None
---------------------------------------------------------------------------*/
void get_path (char *path)
   {
   int pathLength;		// length of pather
   path[0] = 'A' + getdisk();	// 0th char drive name is 'A'
   strcpy (path+1, ":\\");    	// next 2 chars are : and \
   getcurdir (0, path+3);     	// get current path (store after 3 chars)
   pathLength = strlen(path); 	// length of the path
   if (pathLength == 3) strcpy (path+pathLength, "*.*");   // root directory?
   else                 strcpy (path+pathLength, "\\*.*"); // no
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Handle file
INPUT:	        Current path, the selected file number and whether to show
		the image or its info
PROCESS:	Calls show_pcx, show_bmp and show_gif depending on the nature
		of the file selected if the info is 0; else calls
		show_file_info and tells it the type of the image
OUTPUT:		None
---------------------------------------------------------------------------*/
void handle_file (char path[], int &selected, int info)
   {
   filez *temp=head;		// start at first file name
   char filename[80];		// to store file name
   char ext[3];			// temporary extention of the selected file
   int i, start, pathLength;	// length of path in bytes

   pathLength = strlen (path);
   for (i=1; i<selected; i++) temp=temp->next;	// move in the list
   strncpy (filename, path, pathLength-3);	// all except *.* at end
   filename[pathLength-3] = '\0';		// terminate with null char
   strcat (filename, temp->name);		// add file name at the end
   start = strlen (temp->name) - 3;		// start at the extention
   strcpy (ext, temp->name+start);		// copy the rest in ext

   if (strlen (temp->name) > 4 && strcmpi ("BMP", ext) == 0)
      {
      if (info == 0)
	 {
	 i = show_bmp (filename);		// defined in bmp.cpp
	 if (i != 0) error (bmpError[i]);	// not successful?
	 }
      else show_file_info (filename, 0);	// show file info of a bmp
      }

   else if (strlen (temp->name) > 4 && strcmpi ("PCX", ext) == 0)
      {
      if (info == 0)
	 {
	 i = show_pcx (filename);		// deifned in pcx.cpp
	 if (i != 0) error (pcxError[i]);	// not successful?
	 }
      else show_file_info (filename, 1);	// show info of a pcx
      }

   else if (strlen (temp->name) > 4 && strcmpi ("GIF", ext) == 0)
      {
      if (info == 0)
	 {
	 i = show_gif (filename);		// defined in gif.cpp
	 if (i != 0) error (gifError[i]);	// not successful?
	 }
      else show_file_info (filename, 2);	// show info of a gif
      }

   else if (info == 1) return;

   else if (strcmpi (temp->name, "[..]") == 0)	// move to parent directory
      {
      for (i=pathLength-5; i>=0; i--)
	 if (path[i] == '\\') break;		// find last backslash \
      strcpy (path+i+1, "*.*");			// add *.* there
      top = selected = 1;			// reset top and selected
      tFiles = read_directory (path);		// read directory again
      }

   else						// move in a sub directory
      {
      strcpy (path+pathLength-3, temp->name);	// copy dir name into path
      pathLength = strlen (path);		// length of the path
      strcpy (path+pathLength, "\\*.*");	// add *.*
      top = selected = 1;			// reset top and selected
      tFiles = read_directory (path);		// read directory again
      }


   // set back to text mode settings where we left
   _setcursortype (_NOCURSOR);		// make cursor disappear
   draw_page (path);               	// draw title page
   show_directory (head, top, tFiles);  // show the stored directory
   adjust_display (1-top+selected, 0);  // highlight selected file name
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Read directory
INPUT:		Current path (read from get_path)
PROCESS:	Calls findfirst() and findnext() to read in the files and
		stores their names in the linked list pointed by global head
OUTPUT:		Number of file names read
---------------------------------------------------------------------------*/
int read_directory (char *path)
   {
   struct ffblk afile;          // struct to search a file in directory
   filez *tail;                 // end of directory pointer
   char ext[3];			// temporary extention of files found
   int done, pathLength;	// utility variables for this function
   int count = 1;		// number of files read

   while (head != NULL)		// destroy old list
     { tail = head; head = head->next; free (tail); }
   head = new (filez);          // get memory space for directory head
   strcpy (head->name, "[..]"); // store [..] as first file name
   head->next = NULL;           // end linked list

   tail = head;			// make the tail point here too
   done = findfirst (path, &afile, FA_ARCH+FA_DIREC);	// find first file
   while (!done)	        // found first/another?
      {
      if (support (afile))	// add only when the file type is supported
	 {
	 count++;					// increment count
	 tail->next = new (filez);			// get a new tail
	 tail = tail->next;				// move tail there
	 tail->next = NULL;				// make it's next NULL
	 strcpy (tail->name, afile.ff_name);		// copy file name
	 }
      done = findnext (&afile); 			// search next file
      }

   pathLength = strlen (path); 	// are we dealing with a root dir
   if (pathLength == 6)		// root path would be "x:\*.*"
      {
      tail = head;		// shed of the first entry "[..]"
      head = head->next;	// move head one step ahead
      free (tail);		// free up "[..]"
      count--;			// one file less
      }
   return count;		// return number of files read
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Show directory
INPUT:	        Head of the linked list that stores file name and total files
PROCESS:	Calls write() to write file names at fixed spaces
OUTPUT:		None
---------------------------------------------------------------------------*/
void show_directory (filez *head, int top, int tFiles)
   {
   const int TEXT_ATTRIB = 0x1E00;  	// yellow on blue
   const int TITLE_ATTRIB = 0x1F00;	// white on blue
   int i=4, j=4;			// row, col to write file name
   int count, last;			// number of files written

   last = (top+MAXFILES-1 < tFiles) ? top+MAXFILES-1 : tFiles;
   filez *temp;				// pointer to the linked list
   temp = head;				// make it point to the first file
   for (count=1; count<top; count++)
      temp = temp->next;

   for (count=top; count<=last; count++)
      {
      if (i>70) { j++; i=4; }			// width overflow?
      write (i, j, temp->name, TEXT_ATTRIB);	// write file name
      i += 20; temp = temp->next;		// prepare for next file
      }

   if (count < tFiles) write (4, 21, "continued...", TITLE_ATTRIB);
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:       Show information about an image file, called by handle_file
INPUT:          Filename and file type (bmp 0, pcx 1 and gif 2)
PROCESS:        Calls read_pcx_header, read_bmp_header or read_gif_info
                depending upon the file type and uses write() to display a
                part of the header
OUTPUT:		None
---------------------------------------------------------------------------*/
void show_file_info (char *filename, int type)
   {
   const int INFO_ATTRIB  = 0X7100;	// white on blue
   FILE *dataFile;              	// to read header
   pcxHeaderType pcxHeader;		// defined in pcx.cpp
   bmpHeaderType bmpHeader;		// defined in bmp.cpp
   gifInfoType   gifInfo;		// our gif header defined in gif.cpp
   char str[7][80];			// to store strings temporarily
   int i;				// lcv

   dataFile = fopen (filename, "rb");	// open file and check for errors
   if (dataFile == NULL) { error ("Can't read this file"); return; }

   if (type == 0)	// bmp type
      {
      read_bmp_header (dataFile, bmpHeader);		// defined in bmp.cpp
      draw_box (21, 7, 39, 11, INFO_ATTRIB); 		// draw a box
      center_write (7, "[ BMP File Information ]", INFO_ATTRIB);   // title
      sprintf (str[0], "Identity characters:       %c%c", bmpHeader.magic1,
							bmpHeader.magic2);
      sprintf (str[1], "Header size (in bytes):   %3u", bmpHeader.bmiSize);
      sprintf (str[2], "Width:                    %3u", bmpHeader.cols);
      sprintf (str[3], "Height:                   %3u", bmpHeader.rows);
      sprintf (str[4], "Number of color planes:   %3u", bmpHeader.nPlanes);
      sprintf (str[5], "Bits per pixel:           %3u", bmpHeader.bitsPerPixel);
      sprintf (str[6], "Compression:              %3u", bmpHeader.compression);
      }
   else if (type == 1)	// pcx type
      {
      read_pcx_header (dataFile, pcxHeader);		// defined in pcx.cpp
      draw_box (21, 7, 39, 10, INFO_ATTRIB); 		// draw a box
      center_write (7, "[ PCX File Information ]", INFO_ATTRIB); // title
      sprintf (str[0], "Manufacturer:              %3u", pcxHeader.manufacturer);
      sprintf (str[1], "Version:                   %3u", pcxHeader.version);
      sprintf (str[2], "Bits per pixel:            %3u", pcxHeader.bitsPerPixel);
      sprintf (str[3], "Dimensions:          %3u x %3u", pcxHeader.width+1,
						     pcxHeader.height+1);
      sprintf (str[4], "Number of colors planes:   %3u", pcxHeader.nColorPlanes);
      sprintf (str[5], "Palette type:              %3u", pcxHeader.paletteType);
      sprintf (str[6], "");
      }
   else if (type == 2)	// gif type
      {
      read_gif_info (dataFile, gifInfo);                // defined in gif.cpp
      draw_box (21, 7, 39, 10, INFO_ATTRIB);		// draw a box
      center_write (7, "[ GIF File Information ]", INFO_ATTRIB);
      sprintf (str[0], "Version:             %s", gifInfo.version);
      sprintf (str[1], "Global Color Table?     %3u", gifInfo.gcTable);
      sprintf (str[2], "Bits Per Pixel:         %3u", gifInfo.bitsPerPixel);
      sprintf (str[3], "Dimenssions:    %4u x %4u", gifInfo.width, gifInfo.height);
      sprintf (str[4], "Local Color Table?      %3u", gifInfo.lcTable);
      sprintf (str[5], "Image interlaced?       %3u", gifInfo.interlaced);
      sprintf (str[6], "");
      }

   for (i=0; i<7; i++) write (24, 9+i, str[i], INFO_ATTRIB);
   getch();
   fclose (dataFile);
   }
/*-------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------
FUNCTION:	Checks for support of a particular file type
INPUT:	        ffblk file
PROCESS:	Performs several strcmpi's to check extentions etc.
OUTPUT:		1 if supported, 0 otherwise
---------------------------------------------------------------------------*/
int support (ffblk afile)
   {
   char ext[5];
   strcpy (ext, afile.ff_name+strlen (afile.ff_name)-3);
   if      (strcmpi (ext, "BMP") == 0) return 1;
   else if (strcmpi (ext, "GIF") == 0) return 1;
   else if (strcmpi (ext, "PCX") == 0) return 1;
   else if ((afile.ff_attrib & FA_DIREC) != 0)
      {
      if (strcmpi (afile.ff_name, ".") == 0) return 0;
      else if (strcmpi (afile.ff_name, "..") == 0) return 0;
      else return 1;
      }
   else return 0;
   }
/*-------------------------------------------------------------------------*/
